<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\Followupemail2\Model\Event;

use Aheadworks\Followupemail2\Api\Data\EventInterface;
use Aheadworks\Followupemail2\Model\Event\TypeInterface as EventTypeInterface;
use Aheadworks\Followupemail2\Model\Event\TypePool as EventTypePool;
use Aheadworks\Followupemail2\Model\Order\Quote as OrderQuote;
use Aheadworks\Followupemail2\Model\Order\QuoteFactory as OrderQuoteFactory;
use Aheadworks\Followupemail2\Model\ResourceModel\Order\Quote as OrderQuotResource;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Newsletter\Model\ResourceModel\Subscriber\Collection as SubscriberCollection;
use Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory as SubscriberCollectionFactory;
use Magento\Newsletter\Model\Subscriber;
use Magento\Store\Model\App\Emulation as AppEmulation;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\Data\CartInterfaceFactory;
use Magento\Quote\Model\Quote;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\Order;

/**
 * Class Validator
 * @package Aheadworks\Followupemail2\Model\Event
 */
class Validator
{
    /**
     * Const for 'all' value
     */
    const ALL_VALUE = 'all';

    /**
     * @var EventTypePool
     */
    private $eventTypePool;

    /**
     * @var OrderQuotResource
     */
    private $orderQuotResource;

    /**
     * @var OrderQuoteFactory
     */
    private $orderQuoteFactory;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var CartConditionConverter
     */
    private $cartConditionConverter;

    /**
     * @var LifetimeConditionConverter
     */
    private $lifetimeConditionConverter;

    /**
     * @var SubscriberCollectionFactory
     */
    private $subscriberCollectionFactory;

    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * @var CartInterfaceFactory
     */
    private $cartInterfaceFactory;

    /**
     * @var AppEmulation
     */
    private $appEmulation;

    /**
     * @param TypePool $eventTypePool
     * @param OrderQuotResource $orderQuotResource
     * @param OrderQuoteFactory $orderQuoteFactory
     * @param ProductRepositoryInterface $productRepository
     * @param CartConditionConverter $cartConditionConverter
     * @param LifetimeConditionConverter $lifetimeConditionConverter
     * @param SubscriberCollectionFactory $subscriberCollectionFactory
     * @param CartRepositoryInterface $cartRepository
     * @param CartInterfaceFactory $cartInterfaceFactory
     * @param AppEmulation $appEmulation
     */
    public function __construct(
        EventTypePool $eventTypePool,
        OrderQuotResource $orderQuotResource,
        OrderQuoteFactory $orderQuoteFactory,
        ProductRepositoryInterface $productRepository,
        CartConditionConverter $cartConditionConverter,
        LifetimeConditionConverter $lifetimeConditionConverter,
        SubscriberCollectionFactory $subscriberCollectionFactory,
        CartRepositoryInterface $cartRepository,
        CartInterfaceFactory $cartInterfaceFactory,
        AppEmulation $appEmulation
    ) {
        $this->eventTypePool = $eventTypePool;
        $this->orderQuotResource = $orderQuotResource;
        $this->orderQuoteFactory = $orderQuoteFactory;
        $this->productRepository = $productRepository;
        $this->cartConditionConverter = $cartConditionConverter;
        $this->lifetimeConditionConverter = $lifetimeConditionConverter;
        $this->subscriberCollectionFactory = $subscriberCollectionFactory;
        $this->cartRepository = $cartRepository;
        $this->cartInterfaceFactory = $cartInterfaceFactory;
        $this->appEmulation = $appEmulation;
    }

    /**
     * Validate event data by specified event
     *
     * @param EventInterface $event
     * @param array $eventData
     * @param mixed $eventObject
     * @return bool
     */
    public function validate(EventInterface $event, $eventData, $eventObject)
    {
        if (!$this->validateReceiver($event, $eventData)) {
            return false;
        }

        /** @var EventTypeInterface $eventType */
        $eventType = $this->eventTypePool->getType($event->getEventType());

        if (!$this->validateStores($event, $eventData)) {
            return false;
        }

        if ($eventType->isCustomerConditionsEnabled()) {
            if (!$this->validateCustomerGroups($event, $eventData)) {
                return false;
            }

            if (!$this->validateLifetimeCondition($event, $eventData)) {
                return false;
            }
        }

        if ($eventType->isCartConditionsEnabled()) {
            if (!$this->validateCartConditions($event, $eventData, $eventObject)) {
                return false;
            }
        }

        if ($eventType->isOrderConditionsEnabled()) {
            if (!$this->validateOrderCartConditions($event, $eventData, $eventObject)) {
                return false;
            }

            if (!$this->validateOrderStatus($event, $eventData)) {
                return false;
            }
        }

        if ($eventType->isProductConditionsEnabled()) {
            if (!$this->validateProductTypes($event, $eventData, $eventObject)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Validate receiver
     *
     * @param EventInterface $event
     * @param array $eventData
     * @return bool
     */
    private function validateReceiver(EventInterface $event, $eventData)
    {
        if ($event->getNewsletterOnly()) {
            /** @var SubscriberCollection $subscribers */
            $subscribersCollection = $this->subscriberCollectionFactory->create();
            $storeId = $eventData['store_id'];
            $customerEmail = empty($eventData['customer_email']) ? $eventData['email'] : $eventData['customer_email'];
            $subscribersCollection
                ->addStoreFilter($storeId)
                ->addFieldToFilter('subscriber_email', $customerEmail);
            /** @var Subscriber $subscriber */
            $subscriber = $subscribersCollection->getFirstItem();
            if ($subscriber && $subscriber->getSubscriberStatus() == Subscriber::STATUS_SUBSCRIBED) {
                return true;
            }
            return false;
        }
        return true;
    }

    /**
     * Store views validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @return bool
     */
    private function validateStores(EventInterface $event, $eventData)
    {
        if (!in_array($eventData['store_id'], $event->getStoreIds()) &&
            !in_array(0, $event->getStoreIds())
        ) {
            return false;
        }

        return true;
    }

    /**
     * Customer groups validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @return bool
     */
    private function validateCustomerGroups(EventInterface $event, $eventData)
    {
        if (!in_array($eventData['customer_group_id'], $event->getCustomerGroups()) &&
            !in_array(self::ALL_VALUE, $event->getCustomerGroups())
        ) {
            return false;
        }

        return true;
    }

    /**
     * Lifetime condition validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @return bool
     */
    private function validateLifetimeCondition(EventInterface $event, $eventData)
    {
        /** @var LifetimeCondition $lifetimeCondition */
        $lifetimeCondition = $this->lifetimeConditionConverter->getCondition($event);
        if (isset($eventData['customer_id']) && $eventData['customer_id']) {
            return $lifetimeCondition->validate($eventData['customer_id']);
        } else {
            return $lifetimeCondition->validate($eventData['email'], $eventData['store_id']);
        }

        return true;
    }

    /**
     * Order Status validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @return bool
     */
    private function validateOrderStatus(EventInterface $event, $eventData)
    {
        if (!in_array(self::ALL_VALUE, $event->getOrderStatuses())) {
            if (!in_array($eventData['status'], $event->getOrderStatuses())) {
                return false;
            }
        }

        return true;
    }

    /**
     * Product types validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @param mixed $eventObject
     * @return bool
     */
    private function validateProductTypes(EventInterface $event, $eventData, $eventObject)
    {
        if (!in_array(self::ALL_VALUE, $event->getProductTypeIds())) {
            foreach ($eventObject->getItems() as $item) {
                if (!$item->getParentItemId() &&
                    !in_array($item->getProductType(), $event->getProductTypeIds())
                ) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Cart conditions validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @param CartInterface|Quote $eventObject
     * @return bool
     */
    private function validateCartConditions(EventInterface $event, $eventData, $eventObject)
    {
        if ($eventObject) {
            /** @var CartCondition $cartCondition */
            $cartCondition = $this->cartConditionConverter->getCondition($event);

            if ($cartCondition->getConditions()->getConditions()) {
                if ($eventObject->isVirtual()) {
                    $address = $eventObject->getBillingAddress();
                } else {
                    $address = $eventObject->getShippingAddress();
                }

                foreach ($address->getAllItems() as $item) {
                    /** @var ProductInterface|Product $product */
                    $product = $this->productRepository->getById(
                        $item->getProductId(),
                        false,
                        $eventData['store_id'],
                        true
                    );
                    $item->setProduct($product);
                }
                if ($cartCondition->getConditions()->getConditions()) {
                    $eventObject->collectTotals();
                    if (!$cartCondition->validate($address)) {
                        return false;
                    }
                }
            }
            return true;
        }

        return false;
    }

    /**
     * Order conditions validation
     *
     * @param EventInterface $event
     * @param array $eventData
     * @param OrderInterface|Order $eventObject
     * @return bool
     */
    private function validateOrderCartConditions(EventInterface $event, $eventData, $eventObject)
    {
        if ($eventObject) {
            /** @var CartInterface|Quote $cart */
            $cart = $this->getQuote($eventObject->getQuoteId(), $eventObject->getId(), $eventObject->getStoreId());

            /** @var CartCondition $cartCondition */
            $cartCondition = $this->cartConditionConverter->getCondition($event);

            if ($cartCondition->getConditions()->getConditions()) {
                if ($cart->isVirtual()) {
                    $address = $cart->getBillingAddress();
                } else {
                    $address = $cart->getShippingAddress();
                }

                foreach ($address->getAllItems() as $item) {
                    /** @var ProductInterface|Product $product */
                    $product = $this->productRepository->getById(
                        $item->getProductId(),
                        false,
                        $eventData['store_id'],
                        true
                    );
                    $item->setProduct($product);
                }
                if ($cartCondition->getConditions()->getConditions()) {
                    $cart->collectTotals();
                    $address
                        ->setTotalQty($eventObject->getTotalQtyOrdered())
                        ->setPaymentMethod($eventObject->getPayment()->getMethod());
                    if (!$cartCondition->validate($address)) {
                        return false;
                    }
                }
            }
            return true;
        }

        return false;
    }

    /**
     * Get quote
     *
     * @param int $quoteId
     * @param int $orderId
     * @param int $storeId
     * @return CartInterface|Quote
     */
    private function getQuote($quoteId, $orderId, $storeId)
    {
        try {
            $this->appEmulation->startEnvironmentEmulation($storeId, 'frontend', true);
            /** @var CartInterface|Quote $cart */
            $cart = $this->cartRepository->get($quoteId);
            $this->appEmulation->stopEnvironmentEmulation();
        } catch (\Exception $e) {
            /** @var CartInterface|Quote $cart */
            $cart = $this->cartInterfaceFactory->create();

            /** @var OrderQuote $orderQuote */
            $orderQuote = $this->orderQuoteFactory->create();
            $this->orderQuotResource->load($orderQuote, $orderId, 'order_id');

            if ($orderQuote->getId()) {
                $cart->setData($orderQuote->getQuoteData());
            }
        }
        return $cart;
    }
}
